#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
#
# Copyright (C) 2009, Sylvain Raybaud <sylvain.raybaud@crans.org>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of Sylvain Raybaud nor the names of its contributors may
#       be used to endorse or promote products derived from this software
#       without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY Sylvain Raybaud <sylvain.raybaud@crans.org> ``AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Sylvain Raybaud BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#

#if __name__=='__main__':

import sys
import gzip
import os
import random
from math import exp,log
random.seed()

def count_overlap(string,pattern):
    l = len(pattern)
    n = 0
    for i,c in enumerate(string):
        if c==pattern[0] and string[i:i+l]==pattern:
            n += 1
    return n

def open_plain_or_gz(filename,mode='r'):
    """Transparently open gziped or plain files, based on file extension"""
    fname = str(filename)
    return gzip.open(fname,mode) if fname.endswith(".gz") else open(fname,mode)

#backward compatibility
GZ_PLAIN_open = open_plain_or_gz

def pick_random_indices(n,proportion=.5):
    N = (1-proportion)*n-0.5
    indices = range(0,n)
    while N>0:
        indices.remove(random.choice(indices))
        N -= 1
    return indices

def closefiles(files):
    """Close all files in an array"""
    for f in files:
        if f:
            f.close()

def openfiles(filenames,mode="r"):
    """Open all files in an array"""
    files = []
    for f in filenames:
        file = GZ_PLAIN_open(f,mode)
        files.append(file)
    return files

def getbinpath(binname):
    """Given a name look for complete path into PATH environnment variable"""
    path = os.getenv("PATH")
    paths = path.split(os.pathsep)
    for p in paths:
        binpath = p+"/"+binname
        if os.access(binpath,os.X_OK):
            return binpath
    return binname

def verbose_msg(opts,msg):
    if opts.verbose:
        sys.stderr.write(msg+'\n')

def warn(msg):
    """Print a warning message to stderr. The program continues."""
    sys.stderr.write("*** WARNING *** %s\n" % msg)

def error(msg,code=1):
    """Print an error message to stderr and exists with the given code (default: 1)."""
    sys.stderr.write("*** ERROR *** %s\n" % msg)
    exit(code)

def readnlines(stream,n):
    lines = []
    i=0
    while i<n:
        lines.append(stream.readline())
        i += 1
    return lines

def getline(stream,comments=['#'],ignore=["\n"]):
    line = None
    while line!="" and (line==None or (line[0] in comments) or (line in ignore)):
        line = stream.readline()
    return line

def getnlines(stream,n,comments=['#'],ignore=["\n"]):
    lines = []
    i=0
    while i<n:
        lines.append(getline(stream,comments,ignore))
        i += 1
    return lines

log2 = log(2)

def log2(x):
    return log(x)/log2

def log_b(x,b):
    return log(x)/log(b)
